@@ -26,12 +26,13 @@ import json
 from shutil import copyfile
 from uuid import uuid4
 from markupsafe import escape
+from functools import wraps
+
 try:
     from lxml.html.clean import clean_html
 except ImportError:
     pass
 
-
 # Improve this to check if scholarly is available in a global way, like other pythonic libraries
 try:
     from scholarly import scholarly
@@ -52,13 +53,6 @@ from .tasks.upload import TaskUpload
 from .render_template import render_title_template
 from .usermanagement import login_required_if_no_ano
 
-try:
-    from functools import wraps
-except ImportError:
-    pass  # We're not using Python 3
-
-
-
 
 editbook = Blueprint('editbook', __name__)
 log = logger.create()
@@ -67,7 +61,7 @@ log = logger.create()
 def upload_required(f):
     @wraps(f)
     def inner(*args, **kwargs):
-        if current_user.role_upload() or current_user.role_admin():
+        if current_user.role_upload():
             return f(*args, **kwargs)
         abort(403)
 
@@ -238,17 +232,17 @@ def modify_identifiers(input_identifiers, db_identifiers, db_session):
             changed = True
     return changed, error
 
-@editbook.route("/ajax/delete/<int:book_id>")
+@editbook.route("/ajax/delete/<int:book_id>", methods=["POST"])
 @login_required
 def delete_book_from_details(book_id):
-    return Response(delete_book(book_id, "", True), mimetype='application/json')
+    return Response(delete_book_from_table(book_id, "", True), mimetype='application/json')
 
 
-@editbook.route("/delete/<int:book_id>", defaults={'book_format': ""})
-@editbook.route("/delete/<int:book_id>/<string:book_format>")
+@editbook.route("/delete/<int:book_id>", defaults={'book_format': ""}, methods=["POST"])
+@editbook.route("/delete/<int:book_id>/<string:book_format>", methods=["POST"])
 @login_required
 def delete_book_ajax(book_id, book_format):
-    return delete_book(book_id, book_format, False)
+    return delete_book_from_table(book_id, book_format, False)
 
 
 def delete_whole_book(book_id, book):
@@ -318,7 +312,7 @@ def render_delete_book_result(book_format, jsonResponse, warning, book_id):
             return redirect(url_for('web.index'))
 
 
-def delete_book(book_id, book_format, jsonResponse):
+def delete_book_from_table(book_id, book_format, jsonResponse):
     warning = {}
     if current_user.role_delete_books():
         book = calibre_db.get_book(book_id)
@@ -347,6 +341,8 @@ def delete_book(book_id, book_format, jsonResponse):
                 else:
                     calibre_db.session.query(db.Data).filter(db.Data.book == book.id).\
                         filter(db.Data.format == book_format).delete()
+                    if book_format.upper() in ['KEPUB', 'EPUB', 'EPUB3']:
+                        kobo_sync_status.remove_synced_book(book.id, True)
                 calibre_db.session.commit()
             except Exception as ex:
                 log.debug_or_exception(ex)
@@ -363,7 +359,16 @@ def delete_book(book_id, book_format, jsonResponse):
         else:
             # book not found
             log.error('Book with id "%s" could not be deleted: not found', book_id)
-    return render_delete_book_result(book_format, jsonResponse, warning, book_id)
+        return render_delete_book_result(book_format, jsonResponse, warning, book_id)
+    message = _("You are missing permissions to delete books")
+    if jsonResponse:
+        return json.dumps({"location": url_for("editbook.edit_book", book_id=book_id),
+                           "type": "danger",
+                           "format": "",
+                           "message": message})
+    else:
+        flash(message, category="error")
+        return redirect(url_for('editbook.edit_book', book_id=book_id))
 
 
 def render_edit_book(book_id):
@@ -454,7 +459,7 @@ def edit_book_series_index(series_index, book):
     if not series_index.replace('.', '', 1).isdigit():
         flash(_("%(seriesindex)s is not a valid number, skipping", seriesindex=series_index), category="warning")
         return False
-    if book.series_index != series_index:
+    if str(book.series_index) != series_index:
         book.series_index = series_index
         modif_date = True
     return modif_date
@@ -484,11 +489,11 @@ def edit_book_languages(languages, book, upload=False, invalid=None):
     else:
         input_l = isoLanguages.get_valid_language_codes(get_locale(), input_languages, unknown_languages)
     for l in unknown_languages:
-        log.error('%s is not a valid language', l)
+        log.error("'%s' is not a valid language", l)
         if isinstance(invalid, list):
             invalid.append(l)
         else:
-            flash(_(u"%(langname)s is not a valid language", langname=l), category="warning")
+            raise ValueError(_(u"'%(langname)s' is not a valid language", langname=l))
     # ToDo: Not working correct
     if upload and len(input_l) == 1:
         # If the language of the file is excluded from the users view, it's not imported, to allow the user to view
@@ -503,7 +508,7 @@ def edit_book_languages(languages, book, upload=False, invalid=None):
 
 def edit_book_publisher(publishers, book):
     changed = False
-    if publishers:        
+    if publishers:
         publisher = publishers.rstrip().strip()
         if len(book.publishers) == 0 or (len(book.publishers) > 0 and publisher != book.publishers[0].name):
             changed |= modify_database_object([publisher], book.publishers, db.Publishers, calibre_db.session,
@@ -833,7 +838,7 @@ def edit_book(book_id):
 
             if modif_date:
                 book.last_modified = datetime.utcnow()
-                kobo_sync_status.remove_synced_book(edited_books_id)
+                kobo_sync_status.remove_synced_book(edited_books_id, all=True)
 
             calibre_db.session.merge(book)
             calibre_db.session.commit()
@@ -848,6 +853,10 @@ def edit_book(book_id):
             calibre_db.session.rollback()
             flash(error, category="error")
             return render_edit_book(book_id)
+    except ValueError as e:
+        calibre_db.session.rollback()
+        flash(str(e), category="error")
+        return redirect(url_for('web.show_book', book_id=book.id))
     except Exception as ex:
         log.debug_or_exception(ex)
         calibre_db.session.rollback()
@@ -944,7 +953,11 @@ def create_book_on_upload(modif_date, meta):
     modif_date |= edit_book_series_index(meta.series_id, db_book)
 
     # add languages
-    modif_date |= edit_book_languages(meta.languages, db_book, upload=True)
+    invalid=[]
+    modif_date |= edit_book_languages(meta.languages, db_book, upload=True, invalid=invalid)
+    if invalid:
+        for l in invalid:
+            flash(_(u"'%(langname)s' is not a valid language", langname=l), category="warning")
 
     # handle tags
     modif_date |= edit_book_tags(meta.tags, db_book)
@@ -1007,7 +1020,7 @@ def move_coverfile(meta, db_book):
               category="error")
 
 
-@editbook.route("/upload", methods=["GET", "POST"])
+@editbook.route("/upload", methods=["POST"])
 @login_required_if_no_ano
 @upload_required
 def upload():
@@ -1247,7 +1260,7 @@ def merge_list_book():
                                                         element.format,
                                                         element.uncompressed_size,
                                                         to_name))
-                    delete_book(from_book.id,"", True)
+                    delete_book_from_table(from_book.id,"", True)
                     return json.dumps({'success': True})
     return ""
 
