@@ -20,6 +20,20 @@ function getPath() {
     return jsFileLocation.substr(0, jsFileLocation.search("/static/js/libs/jquery.min.js"));  // the js folder path
 }
 
+function postButton(event, action){
+    event.preventDefault();
+    var newForm = jQuery('<form>', {
+        "action": action,
+        'target': "_top",
+        'method': "post"
+    }).append(jQuery('<input>', {
+        'name': 'csrf_token',
+        'value': $("input[name=\'csrf_token\']").val(),
+        'type': 'hidden'
+    })).appendTo('body');
+    newForm.submit();
+}
+
 function elementSorter(a, b) {
     a = +a.slice(0, -2);
     b = +b.slice(0, -2);
@@ -71,6 +85,22 @@ $(document).on("change", "select[data-controlall]", function() {
     }
 });
 
+/*$(document).on("click", "#sendbtn", function (event) {
+    postButton(event, $(this).data('action'));
+});
+
+$(document).on("click", ".sendbutton", function (event) {
+    // $(".sendbutton").on("click", "body", function(event) {
+    postButton(event, $(this).data('action'));
+});*/
+
+$(document).on("click", ".postAction", function (event) {
+    // $(".sendbutton").on("click", "body", function(event) {
+    postButton(event, $(this).data('action'));
+});
+
+
+
 // Syntax has to be bind not on, otherwise problems with firefox
 $(".container-fluid").bind("dragenter dragover", function () {
     if($("#btn-upload").length && !$('body').hasClass('shelforder')) {
@@ -168,18 +198,18 @@ function confirmDialog(id, dialogid, dataValue, yesFn, noFn) {
     $confirm.modal('show');
 }
 
-$("#delete_confirm").click(function() {
+$("#delete_confirm").click(function(event) {
     //get data-id attribute of the clicked element
     var deleteId = $(this).data("delete-id");
     var bookFormat = $(this).data("delete-format");
     var ajaxResponse = $(this).data("ajax");
     if (bookFormat) {
-        window.location.href = getPath() + "/delete/" + deleteId + "/" + bookFormat;
+        postButton(event, getPath() + "/delete/" + deleteId + "/" + bookFormat);
     } else {
         if (ajaxResponse) {
             path = getPath() + "/ajax/delete/" + deleteId;
             $.ajax({
-                method:"get",
+                method:"post",
                 url: path,
                 timeout: 900,
                 success:function(data) {
@@ -198,8 +228,7 @@ $("#delete_confirm").click(function() {
                 }
             });
         } else {
-            window.location.href = getPath() + "/delete/" + deleteId;
-
+            postButton(event, getPath() + "/delete/" + deleteId);
         }
     }
 
@@ -284,11 +313,7 @@ $(function() {
     }
 
     function fillFileTable(path, type, folder, filt) {
-        if (window.location.pathname.endsWith("/basicconfig")) {
-            var request_path = "/../basicconfig/pathchooser/";
-        } else {
-            var request_path = "/../../ajax/pathchooser/";
-        }
+        var request_path = "/../../ajax/pathchooser/";
         $.ajax({
             dataType: "json",
             data: {
@@ -380,9 +405,11 @@ $(function() {
 
     $("#restart").click(function() {
         $.ajax({
+            method:"post",
+            contentType: "application/json; charset=utf-8",
             dataType: "json",
-            url: window.location.pathname + "/../../shutdown",
-            data: {"parameter":0},
+            url: getPath() + "/shutdown",
+            data: JSON.stringify({"parameter":0}),
             success: function success() {
                 $("#spinner").show();
                 setTimeout(restartTimer, 3000);
@@ -391,9 +418,11 @@ $(function() {
     });
     $("#shutdown").click(function() {
         $.ajax({
+            method:"post",
+            contentType: "application/json; charset=utf-8",
             dataType: "json",
-            url: window.location.pathname + "/../../shutdown",
-            data: {"parameter":1},
+            url: getPath() + "/shutdown",
+            data: JSON.stringify({"parameter":1}),
             success: function success(data) {
                 return alert(data.text);
             }
@@ -451,9 +480,11 @@ $(function() {
         $("#DialogContent").html("");
         $("#spinner2").show();
         $.ajax({
+            method:"post",
+            contentType: "application/json; charset=utf-8",
             dataType: "json",
             url: getPath() + "/shutdown",
-            data: {"parameter":2},
+            data: JSON.stringify({"parameter":2}),
             success: function success(data) {
                 $("#spinner2").hide();
                 $("#DialogContent").html(data.text);
@@ -504,6 +535,7 @@ $(function() {
 
     $("#modal_kobo_token")
         .on("show.bs.modal", function(e) {
+            $(e.relatedTarget).one('focus', function(e){$(this).blur();});
             var $modalBody = $(this).find(".modal-body");
 
             // Prevent static assets from loading multiple times
@@ -521,6 +553,7 @@ $(function() {
         .on("hidden.bs.modal", function() {
             $(this).find(".modal-body").html("...");
             $("#config_delete_kobo_token").show();
+            $("#kobo_full_sync").show();
         });
 
     $("#config_delete_kobo_token").click(function() {
@@ -530,10 +563,11 @@ $(function() {
             $(this).data('value'),
             function (value) {
                 $.ajax({
-                    method: "get",
+                    method: "post",
                     url: getPath() + "/kobo_auth/deleteauthtoken/" + value,
                 });
                 $("#config_delete_kobo_token").hide();
+                $("#kobo_full_sync").hide();
             }
         );
     });
@@ -567,6 +601,33 @@ $(function() {
             }
         );
     });
+
+    $("#kobo_full_sync").click(function() {
+        confirmDialog(
+           "btnfullsync",
+            "GeneralDeleteModal",
+            $(this).data('value'),
+            function(value){
+                path = getPath() + "/ajax/fullsync"
+                $.ajax({
+                    method:"post",
+                    url: path,
+                    timeout: 900,
+                    success:function(data) {
+                        data.forEach(function(item) {
+                            if (!jQuery.isEmptyObject(item)) {
+                                $( ".navbar" ).after( '<div class="row-fluid text-center" >' +
+                                    '<div id="flash_'+item.type+'" class="alert alert-'+item.type+'">'+item.message+'</div>' +
+                                    '</div>');
+                            }
+                        });
+                    }
+                });
+            }
+        );
+    });
+
+
     $("#user_submit").click(function() {
         this.closest("form").submit();
     });
@@ -613,7 +674,7 @@ $(function() {
                     else {
                         $("#InvalidDialog").modal('show');
                     }
-                } else {                	
+                } else {
                     changeDbSettings();
                 }
             }
@@ -654,13 +715,14 @@ $(function() {
         });
     });
 
-    $("#delete_shelf").click(function() {
+    $("#delete_shelf").click(function(event) {
         confirmDialog(
             $(this).attr('id'),
             "GeneralDeleteModal",
             $(this).data('value'),
             function(value){
-                window.location.href = window.location.pathname + "/../../shelf/delete/" + value
+                postButton(event, $("#delete_shelf").data("action"));
+                // $("#delete_shelf").closest("form").submit()
             }
         );
 
@@ -709,7 +771,8 @@ $(function() {
         $("#DialogContent").html("");
         $("#spinner2").show();
         $.ajax({
-            method:"get",
+            method:"post",
+            contentType: "application/json; charset=utf-8",
             dataType: "json",
             url: getPath() + "/import_ldap_users",
             success: function success(data) {
@@ -743,4 +806,3 @@ $(function() {
         });
     });
 });
-
