@@ -54,7 +54,9 @@ from .helper import check_valid_domain, render_task_status, check_email, check_u
 from .pagination import Pagination
 from .redirect import redirect_back
 from .usermanagement import login_required_if_no_ano
+from .kobo_sync_status import remove_synced_book
 from .render_template import render_title_template
+from .kobo_sync_status import change_archived_books
 
 feature_support = {
     'ldap': bool(services.ldap),
@@ -85,6 +87,8 @@ def add_security_headers(resp):
     resp.headers['Content-Security-Policy'] = "default-src 'self'" + ''.join([' '+host for host in config.config_trustedhosts.strip().split(',')]) + " 'unsafe-inline' 'unsafe-eval'; font-src 'self' data:; img-src 'self' data:"
     if request.endpoint == "editbook.edit_book" or config.config_use_google_drive:
         resp.headers['Content-Security-Policy'] += " *"
+    elif request.endpoint == "web.read_book":
+        resp.headers['Content-Security-Policy'] += " blob:;style-src-elem 'self' blob: 'unsafe-inline';"
     resp.headers['X-Content-Type-Options'] = 'nosniff'
     resp.headers['X-Frame-Options'] = 'SAMEORIGIN'
     resp.headers['X-XSS-Protection'] = '1; mode=block'
@@ -187,30 +191,23 @@ def toggle_read(book_id):
             return "Custom Column No.{} is not existing in calibre database".format(config.config_read_column), 400
         except (OperationalError, InvalidRequestError) as e:
             calibre_db.session.rollback()
-            log.error(u"Read status could not set: %e", e)
+            log.error(u"Read status could not set: {}".format(e))
             return "Read status could not set: {}".format(e), 400
     return ""
 
 @web.route("/ajax/togglearchived/<int:book_id>", methods=['POST'])
 @login_required
 def toggle_archived(book_id):
-    archived_book = ub.session.query(ub.ArchivedBook).filter(and_(ub.ArchivedBook.user_id == int(current_user.id),
-                                                                  ub.ArchivedBook.book_id == book_id)).first()
-    if archived_book:
-        archived_book.is_archived = not archived_book.is_archived
-        archived_book.last_modified = datetime.utcnow()
-    else:
-        archived_book = ub.ArchivedBook(user_id=current_user.id, book_id=book_id)
-        archived_book.is_archived = True
-    ub.session.merge(archived_book)
-    ub.session_commit("Book {} archivebit toggled".format(book_id))
+    is_archived = change_archived_books(book_id, message="Book {} archivebit toggled".format(book_id))
+    if is_archived:
+        remove_synced_book(book_id)
     return ""
 
 
 @web.route("/ajax/view", methods=["POST"])
 @login_required_if_no_ano
 def update_view():
-    to_save = request.form.to_dict() # request.get_json()
+    to_save = request.get_json()
     try:
         for element in to_save:
             for param in to_save[element]:
@@ -362,7 +359,13 @@ def get_sort_function(sort, data):
         order = [db.Books.series_index.asc()]
     if sort == 'seriesdesc':
         order = [db.Books.series_index.desc()]
-    return order
+    if sort == 'hotdesc':
+        order = [func.count(ub.Downloads.book_id).desc()]
+    if sort == 'hotasc':
+        order = [func.count(ub.Downloads.book_id).asc()]
+    if sort is None:
+        sort = "new"
+    return order, sort
 
 
 def render_books_list(data, sort, book_id, page):
@@ -376,7 +379,7 @@ def render_books_list(data, sort, book_id, page):
     elif data == "read":
         return render_read_books(page, True, order=order)
     elif data == "hot":
-        return render_hot_books(page)
+        return render_hot_books(page, order)
     elif data == "download":
         return render_downloaded_books(page, order, book_id)
     elif data == "author":
@@ -405,12 +408,12 @@ def render_books_list(data, sort, book_id, page):
         return render_adv_search_results(term, offset, order, config.config_books_per_page)
     else:
         website = data or "newest"
-        entries, random, pagination = calibre_db.fill_indexpage(page, 0, db.Books, True, order,
+        entries, random, pagination = calibre_db.fill_indexpage(page, 0, db.Books, True, order[0],
                                                                 db.books_series_link,
                                                                 db.Books.id == db.books_series_link.c.book,
                                                                 db.Series)
         return render_title_template('index.html', random=random, entries=entries, pagination=pagination,
-                                     title=_(u"Books"), page=website)
+                                     title=_(u"Books"), page=website, order=order[1])
 
 
 def render_rated_books(page, book_id, order):
@@ -418,13 +421,13 @@ def render_rated_books(page, book_id, order):
         entries, random, pagination = calibre_db.fill_indexpage(page, 0,
                                                                 db.Books,
                                                                 db.Books.ratings.any(db.Ratings.rating > 9),
-                                                                order,
+                                                                order[0],
                                                                 db.books_series_link,
                                                                 db.Books.id == db.books_series_link.c.book,
                                                                 db.Series)
 
         return render_title_template('index.html', random=random, entries=entries, pagination=pagination,
-                                     id=book_id, title=_(u"Top Rated Books"), page="rated")
+                                     id=book_id, title=_(u"Top Rated Books"), page="rated", order=order[1])
     else:
         abort(404)
 
@@ -438,16 +441,21 @@ def render_discover_books(page, book_id):
     else:
         abort(404)
 
-def render_hot_books(page):
+def render_hot_books(page, order):
     if current_user.check_visibility(constants.SIDEBAR_HOT):
+        if order[1] not in ['hotasc', 'hotdesc']:
+        # Unary expression comparsion only working (for this expression) in sqlalchemy 1.4+
+        #if not (order[0][0].compare(func.count(ub.Downloads.book_id).desc()) or
+        #        order[0][0].compare(func.count(ub.Downloads.book_id).asc())):
+            order = [func.count(ub.Downloads.book_id).desc()], 'hotdesc'
         if current_user.show_detail_random():
             random = calibre_db.session.query(db.Books).filter(calibre_db.common_filters()) \
                 .order_by(func.random()).limit(config.config_random_books)
         else:
             random = false()
         off = int(int(config.config_books_per_page) * (page - 1))
-        all_books = ub.session.query(ub.Downloads, func.count(ub.Downloads.book_id)).order_by(
-            func.count(ub.Downloads.book_id).desc()).group_by(ub.Downloads.book_id)
+        all_books = ub.session.query(ub.Downloads, func.count(ub.Downloads.book_id))\
+            .order_by(*order[0]).group_by(ub.Downloads.book_id)
         hot_books = all_books.offset(off).limit(config.config_books_per_page)
         entries = list()
         for book in hot_books:
@@ -460,7 +468,7 @@ def render_hot_books(page):
         numBooks = entries.__len__()
         pagination = Pagination(page, config.config_books_per_page, numBooks)
         return render_title_template('index.html', random=random, entries=entries, pagination=pagination,
-                                     title=_(u"Hot Books (Most Downloaded)"), page="hot")
+                                     title=_(u"Hot Books (Most Downloaded)"), page="hot", order=order[1])
     else:
         abort(404)
 
@@ -481,7 +489,10 @@ def render_downloaded_books(page, order, user_id):
                                                             0,
                                                             db.Books,
                                                             ub.Downloads.user_id == user_id,
-                                                            order,
+                                                            order[0],
+                                                            db.books_series_link,
+                                                            db.Books.id == db.books_series_link.c.book,
+                                                            db.Series,
                                                             ub.Downloads, db.Books.id == ub.Downloads.book_id)
         for book in entries:
             if not calibre_db.session.query(db.Books).filter(calibre_db.common_filters()) \
@@ -494,7 +505,8 @@ def render_downloaded_books(page, order, user_id):
                                      pagination=pagination,
                                      id=user_id,
                                      title=_(u"Downloaded books by %(user)s",user=user.name),
-                                     page="download")
+                                     page="download",
+                                     order=order[1])
     else:
         abort(404)
 
@@ -503,7 +515,7 @@ def render_author_books(page, author_id, order):
     entries, __, pagination = calibre_db.fill_indexpage(page, 0,
                                                         db.Books,
                                                         db.Books.authors.any(db.Authors.id == author_id),
-                                                        [order[0], db.Series.name, db.Books.series_index],
+                                                        [order[0][0], db.Series.name, db.Books.series_index],
                                                         db.books_series_link,
                                                         db.Books.id == db.books_series_link.c.book,
                                                         db.Series)
@@ -525,7 +537,7 @@ def render_author_books(page, author_id, order):
 
     return render_title_template('author.html', entries=entries, pagination=pagination, id=author_id,
                                  title=_(u"Author: %(name)s", name=author_name), author=author_info,
-                                 other_books=other_books, page="author")
+                                 other_books=other_books, page="author", order=order[1])
 
 
 def render_publisher_books(page, book_id, order):
@@ -534,12 +546,14 @@ def render_publisher_books(page, book_id, order):
         entries, random, pagination = calibre_db.fill_indexpage(page, 0,
                                                                 db.Books,
                                                                 db.Books.publishers.any(db.Publishers.id == book_id),
-                                                                [db.Series.name, order[0], db.Books.series_index],
+                                                                [db.Series.name, order[0][0], db.Books.series_index],
                                                                 db.books_series_link,
                                                                 db.Books.id == db.books_series_link.c.book,
                                                                 db.Series)
         return render_title_template('index.html', random=random, entries=entries, pagination=pagination, id=book_id,
-                                     title=_(u"Publisher: %(name)s", name=publisher.name), page="publisher")
+                                     title=_(u"Publisher: %(name)s", name=publisher.name),
+                                     page="publisher",
+                                     order=order[1])
     else:
         abort(404)
 
@@ -550,9 +564,9 @@ def render_series_books(page, book_id, order):
         entries, random, pagination = calibre_db.fill_indexpage(page, 0,
                                                                 db.Books,
                                                                 db.Books.series.any(db.Series.id == book_id),
-                                                                [order[0]])
+                                                                [order[0][0]])
         return render_title_template('index.html', random=random, pagination=pagination, entries=entries, id=book_id,
-                                     title=_(u"Series: %(serie)s", serie=name.name), page="series")
+                                     title=_(u"Series: %(serie)s", serie=name.name), page="series", order=order[1])
     else:
         abort(404)
 
@@ -562,10 +576,12 @@ def render_ratings_books(page, book_id, order):
     entries, random, pagination = calibre_db.fill_indexpage(page, 0,
                                                             db.Books,
                                                             db.Books.ratings.any(db.Ratings.id == book_id),
-                                                            [order[0]])
+                                                            [order[0][0]])
     if name and name.rating <= 10:
         return render_title_template('index.html', random=random, pagination=pagination, entries=entries, id=book_id,
-                                     title=_(u"Rating: %(rating)s stars", rating=int(name.rating / 2)), page="ratings")
+                                     title=_(u"Rating: %(rating)s stars", rating=int(name.rating / 2)),
+                                     page="ratings",
+                                     order=order[1])
     else:
         abort(404)
 
@@ -576,9 +592,11 @@ def render_formats_books(page, book_id, order):
         entries, random, pagination = calibre_db.fill_indexpage(page, 0,
                                                                 db.Books,
                                                                 db.Books.data.any(db.Data.format == book_id.upper()),
-                                                                [order[0]])
+                                                                [order[0][0]])
         return render_title_template('index.html', random=random, pagination=pagination, entries=entries, id=book_id,
-                                     title=_(u"File format: %(format)s", format=name.format), page="formats")
+                                     title=_(u"File format: %(format)s", format=name.format),
+                                     page="formats",
+                                     order=order[1])
     else:
         abort(404)
 
@@ -589,12 +607,12 @@ def render_category_books(page, book_id, order):
         entries, random, pagination = calibre_db.fill_indexpage(page, 0,
                                                                 db.Books,
                                                                 db.Books.tags.any(db.Tags.id == book_id),
-                                                                [order[0], db.Series.name, db.Books.series_index],
+                                                                [order[0][0], db.Series.name, db.Books.series_index],
                                                                 db.books_series_link,
                                                                 db.Books.id == db.books_series_link.c.book,
                                                                 db.Series)
         return render_title_template('index.html', random=random, entries=entries, pagination=pagination, id=book_id,
-                                     title=_(u"Category: %(name)s", name=name.name), page="category")
+                                     title=_(u"Category: %(name)s", name=name.name), page="category", order=order[1])
     else:
         abort(404)
 
@@ -608,13 +626,13 @@ def render_language_books(page, name, order):
     entries, random, pagination = calibre_db.fill_indexpage(page, 0,
                                                             db.Books,
                                                             db.Books.languages.any(db.Languages.lang_code == name),
-                                                            [order[0]])
+                                                            [order[0][0]])
     return render_title_template('index.html', random=random, entries=entries, pagination=pagination, id=name,
-                                 title=_(u"Language: %(name)s", name=lang_name), page="language")
+                                 title=_(u"Language: %(name)s", name=lang_name), page="language", order=order[1])
 
 
 def render_read_books(page, are_read, as_xml=False, order=None):
-    order = order or []
+    sort = order[0] if order else []
     if not config.config_read_column:
         if are_read:
             db_filter = and_(ub.ReadBook.user_id == int(current_user.id),
@@ -624,7 +642,7 @@ def render_read_books(page, are_read, as_xml=False, order=None):
         entries, random, pagination = calibre_db.fill_indexpage(page, 0,
                                                                 db.Books,
                                                                 db_filter,
-                                                                order,
+                                                                sort,
                                                                 db.books_series_link,
                                                                 db.Books.id == db.books_series_link.c.book,
                                                                 db.Series,
@@ -638,7 +656,7 @@ def render_read_books(page, are_read, as_xml=False, order=None):
             entries, random, pagination = calibre_db.fill_indexpage(page, 0,
                                                                     db.Books,
                                                                     db_filter,
-                                                                    order,
+                                                                    sort,
                                                                     db.books_series_link,
                                                                     db.Books.id == db.books_series_link.c.book,
                                                                     db.Series,
@@ -661,11 +679,11 @@ def render_read_books(page, are_read, as_xml=False, order=None):
             name = _(u'Unread Books') + ' (' + str(pagination.total_count) + ')'
             pagename = "unread"
         return render_title_template('index.html', random=random, entries=entries, pagination=pagination,
-                                     title=name, page=pagename)
+                                     title=name, page=pagename, order=order[1])
 
 
-def render_archived_books(page, order):
-    order = order or []
+def render_archived_books(page, sort):
+    order = sort[0] or []
     archived_books = (
         ub.session.query(ub.ArchivedBook)
         .filter(ub.ArchivedBook.user_id == int(current_user.id))
@@ -685,7 +703,7 @@ def render_archived_books(page, order):
     name = _(u'Archived Books') + ' (' + str(len(archived_book_ids)) + ')'
     pagename = "archived"
     return render_title_template('index.html', random=random, entries=entries, pagination=pagination,
-                                 title=name, page=pagename)
+                                 title=name, page=pagename, order=sort[1])
 
 
 def render_prepare_search_form(cc):
@@ -730,7 +748,8 @@ def render_search_results(term, offset=None, order=None, limit=None):
                                  entries=entries,
                                  result_count=result_count,
                                  title=_(u"Search"),
-                                 page="search")
+                                 page="search",
+                                 order=order[1])
 
 
 # ################################### View Books list ##################################################################
@@ -774,7 +793,6 @@ def list_books():
 
     if sort == "state":
         state = json.loads(request.args.get("state", "[]"))
-        # order = [db.Books.timestamp.asc()] if order == "asc" else [db.Books.timestamp.desc()]   # ToDo wrong: sort ticked
     elif sort == "tags":
         order = [db.Tags.name.asc()] if order == "asc" else [db.Tags.name.desc()]
         join = db.books_tags_link,db.Books.id == db.books_tags_link.c.book, db.Tags
@@ -807,7 +825,7 @@ def list_books():
             books = calibre_db.session.query(db.Books).filter(calibre_db.common_filters()).all()
         entries = calibre_db.get_checkbox_sorted(books, state, off, limit, order)
     elif search:
-        entries, filtered_count, __ = calibre_db.get_search_results(search, off, order, limit, *join)
+        entries, filtered_count, __ = calibre_db.get_search_results(search, off, [order,''], limit, *join)
     else:
         entries, __, __ = calibre_db.fill_indexpage((int(off) / (int(limit)) + 1), limit, db.Books, True, order, *join)
 
@@ -927,10 +945,11 @@ def series_list():
                 .join(db.books_series_link).join(db.Books).filter(calibre_db.common_filters()) \
                 .group_by(func.upper(func.substr(db.Series.sort, 1, 1))).all()
             return render_title_template('list.html', entries=entries, folder='web.books_list', charlist=charlist,
-                                         title=_(u"Series"), page="serieslist", data="series")
+                                         title=_(u"Series"), page="serieslist", data="series", order=order_no)
         else:
-            entries = calibre_db.session.query(db.Books, func.count('books_series_link').label('count')) \
-                .join(db.books_series_link).join(db.Series).filter(calibre_db.common_filters()) \
+            entries = calibre_db.session.query(db.Books, func.count('books_series_link').label('count'),
+                                               func.max(db.Books.series_index), db.Books.id) \
+                .join(db.books_series_link).join(db.Series).filter(calibre_db.common_filters())\
                 .group_by(text('books_series_link.series')).order_by(order).all()
             charlist = calibre_db.session.query(func.upper(func.substr(db.Series.sort, 1, 1)).label('char')) \
                 .join(db.books_series_link).join(db.Books).filter(calibre_db.common_filters()) \
@@ -987,30 +1006,17 @@ def formats_list():
 @web.route("/language")
 @login_required_if_no_ano
 def language_overview():
-    if current_user.check_visibility(constants.SIDEBAR_LANGUAGE):
+    if current_user.check_visibility(constants.SIDEBAR_LANGUAGE) and current_user.filter_language() == u"all":
+        order_no = 0 if current_user.get_view_property('language', 'dir') == 'desc' else 1
         charlist = list()
-        if current_user.filter_language() == u"all":
-            languages = calibre_db.speaking_language()
-            # ToDo: generate first character list for languages
-        else:
-            #try:
-            #    cur_l = LC.parse(current_user.filter_language())
-            #except UnknownLocaleError:
-            #    cur_l = None
-
-            languages = calibre_db.session.query(db.Languages).filter(
-                db.Languages.lang_code == current_user.filter_language()).all()
-            languages[0].name = isoLanguages.get_language_name(get_locale(), languages[0].name.lang_code)
-            #if cur_l:
-            #    languages[0].name = cur_l.get_language_name(get_locale())
-            #else:
-            #    languages[0].name = _(isoLanguages.get(part3=languages[0].lang_code).name)
-        lang_counter = calibre_db.session.query(db.books_languages_link,
-                                        func.count('books_languages_link.book').label('bookcount')).group_by(
-            text('books_languages_link.lang_code')).all()
-        return render_title_template('languages.html', languages=languages, lang_counter=lang_counter,
+        languages = calibre_db.speaking_language(reverse_order=not order_no, with_count=True)
+        for lang in languages:
+            upper_lang = lang[0].name[0].upper()
+            if upper_lang not in charlist:
+                charlist.append(upper_lang)
+        return render_title_template('languages.html', languages=languages,
                                      charlist=charlist, title=_(u"Languages"), page="langlist",
-                                     data="language")
+                                     data="language", order=order_no)
     else:
         abort(404)
 
@@ -1049,7 +1055,8 @@ def get_tasks_status():
     return render_title_template('tasks.html', entries=answer, title=_(u"Tasks"), page="tasks")
 
 
-@app.route("/reconnect")
+# method is available without login and not protected by CSRF to make it easy reachable
+@app.route("/reconnect", methods=['GET'])
 def reconnect():
     calibre_db.reconnect_db(config, ub.app_DB_path)
     return json.dumps({})
@@ -1062,7 +1069,7 @@ def reconnect():
 def search():
     term = request.args.get("query")
     if term:
-        return redirect(url_for('web.books_list', data="search", sort_param='stored', query=term))
+        return redirect(url_for('web.books_list', data="search", sort_param='stored', query=term.strip()))
     else:
         return render_title_template('search.html',
                                      searchterm="",
@@ -1247,7 +1254,7 @@ def extend_search_term(searchterm,
 
 
 def render_adv_search_results(term, offset=None, order=None, limit=None):
-    order = order or [db.Books.sort]
+    sort = order[0] if order else [db.Books.sort]
     pagination = None
 
     cc = get_cc_columns(filter_config_custom_read=True)
@@ -1345,7 +1352,7 @@ def render_adv_search_results(term, offset=None, order=None, limit=None):
             log.debug_or_exception(ex)
             flash(_("Error on search for custom columns, please restart Calibre-Web"), category="error")
 
-    q = q.order_by(*order).all()
+    q = q.order_by(*sort).all()
     flask_session['query'] = json.dumps(term)
     ub.store_ids(q)
     result_count = len(q)
@@ -1361,7 +1368,8 @@ def render_adv_search_results(term, offset=None, order=None, limit=None):
                                  pagination=pagination,
                                  entries=q[offset:limit_all],
                                  result_count=result_count,
-                                 title=_(u"Advanced Search"), page="advsearch")
+                                 title=_(u"Advanced Search"), page="advsearch",
+                                 order=order[1])
 
 
 
@@ -1428,7 +1436,7 @@ def download_link(book_id, book_format, anyname):
     return get_download_link(book_id, book_format, client)
 
 
-@web.route('/send/<int:book_id>/<book_format>/<int:convert>')
+@web.route('/send/<int:book_id>/<book_format>/<int:convert>', methods=["POST"])
 @login_required
 @download_required
 def send_to_kindle(book_id, book_format, convert):
@@ -1509,9 +1517,6 @@ def register():
 
 @web.route('/login', methods=['GET', 'POST'])
 def login():
-    #if not config.db_configured:
-    #    log.debug(u"Redirect to initial configuration")
-    #    return redirect(url_for('admin.basic_configuration'))
     if current_user is not None and current_user.is_authenticated:
         return redirect(url_for('web.index'))
     if config.config_login_type == constants.LOGIN_LDAP and not services.ldap:
